#ifndef QUATERNION_H_
#define QUATERNION_H_

#include <iostream>
#include "vector.h"

/* Quaternion */
class Quaternion {
public:
	float s;
	Vector3 v;

	Quaternion();
	Quaternion(float s, const Vector3 &v);
	Quaternion(float s, float x, float y, float z);
	Quaternion(const Vector3 &axis, float angle);

	Quaternion operator +(const Quaternion &quat) const;
	Quaternion operator -(const Quaternion &quat) const;
	Quaternion operator -() const;
	Quaternion operator *(const Quaternion &quat) const;
	
	void operator +=(const Quaternion &quat);
	void operator -=(const Quaternion &quat);
	void operator *=(const Quaternion &quat);

	void ResetIdentity();

	Quaternion Conjugate() const;

	float Length() const;
	float LengthSq() const;
	
	void Normalize();
	Quaternion Normalized() const;

	Quaternion Inverse() const;

	void SetRotation(const Vector3 &axis, float angle);
	void Rotate(const Vector3 &axis, float angle);
	/* note: this is a totally different operation from the above
	 * this treats the quaternion as signifying direction and rotates
	 * it by a rotation quaternion by rot * q * rot'
	 */
	void Rotate(const Quaternion &q);

	Matrix3x3 GetRotationMatrix() const;
	
	friend Quaternion Slerp(const Quaternion &q1, const Quaternion &q2, float t);
	
	friend std::ostream &operator <<(std::ostream &out, const Quaternion &q);
};


#endif	// QUATERNION_H_
